#include <iostream>
#include "../range.hpp"

// everything belongs to range is in Range namespace
using namespace Range;

int main()
{
    /*
        A range is a sequence of numeric elements.
        So, when you declare a range, you specify
        the first element(inclusive), the last
        element(exclusive) and finally how much
        difference is there between each two
        successive elements:
        range x(first_element = 1,
                last_element  = 10,
                step_size     = 2);
        The range would be like: [1, 10), [step = 2]
        The elements are:        [1, 3, 5, 7, 9]
    */
    range x(1, 10, 2);

    // you can use the range as if it were an array!
    range::value_type sum_of_range_elements = 0;
    for(std::size_t i = 0; i < x.size(); ++i)
    {
        sum_of_range_elements += x[i];
    }

    // you can go with STL-style iterators!
    range::value_type product_of_range_elements = 1;
    for(range::iterator i = x.begin(); i != x.end(); ++i)
    {
        product_of_range_elements *= *i;
    }

    // printing the elements of the range
    std::cout << "The range x(1, 10, 2) consists of: [ ";
    for(range::iterator i = x.begin(); i != x.end(); ++i)
    {
        std::cout << *i << ' ';
    }
    std::cout << ']' << std::endl;

    std::cout << "The sum of x elements = "
        << sum_of_range_elements << std::endl;
    std::cout << "The product of x elements = "
        << product_of_range_elements << std::endl;
}