#include <iostream>
#include <vector>
#include <algorithm>
#include <numeric>
#include "../range.hpp"

// everything belongs to range is in Range namespace
using namespace Range;

/*
    range works with standards algorithms as well!
    range is immutable, i.e. you can't apply
    modifiying algorithms on it, only the
    non-modifiying algorithms can be applied.
*/

// A simple product functor. We will use it
// with for_each
struct functor
{
    functor() : product(1) { }
    void operator()(int number)
    {
        product *= number;
    }
    int product;
};

// to be used with count_if
bool is_odd(int number)
{
    return number % 2 == 1;
}

int main()
{
    range r(1, 10, 1); // [1, 2, 3, 4, 5, 6, 7, 8, 9]

    // with for_each
    int product
        = std::for_each( r.begin(), r.end(), functor() ).product;
    std::cout 
        << "the product of the range elements = "
        << product << std::endl;

    std::cout << std::endl;

    // with count_if
    int count_odd_numbers
        = std::count_if( r.begin(), r.end(), is_odd );
    std::cout
        << "there are " << count_odd_numbers
        << " odd numbers in the range" << std::endl;

    std::cout << std::endl;

    // with accumulate
    // sum = 1000 + (range elements summation)
    int sum
        = std::accumulate( r.begin(), r.end(), 1000 );
    std::cout
        << "the sum of the range elements + 1000 = "
        << sum << std::endl;

    std::cout << std::endl;

    // with partial_sum
    std::vector<int> partial_summation( r.size() );
    std::partial_sum( r.begin(), r.end(), partial_summation.begin() );
    for(std::size_t i = 0; i < partial_summation.size(); ++i)
        std::cout
            << "partial summation until " << r[i]
            << " = " << partial_summation[i] << std::endl;

    std::cout << std::endl;
}